@@ -18,7 +18,7 @@
 #
 #    (C) 2005 Nicolas Évrard <nicoe@nutellux.be>
 
-__revision__ = "$Id: gnomolicious.py,v 1.17 2006/01/12 01:30:53 nicoe Exp $"
+__revision__ = "$Id: gnomolicious.py,v 1.18 2006/01/13 15:04:53 nicoe Exp $"
 
 import os.path
 import re
@@ -110,6 +110,8 @@ class GnomoliciousConfiguration(object):
 
 class GnomoliciousApplet(gnomeapplet.Applet):
 
+    charset_re = re.compile('<meta[^>]*content="[^"]*charset=(?P<enc>[\w-]*)"',
+                            re.IGNORECASE|re.DOTALL)
     title_re = re.compile('<title>(?P<title>.*?)</title>',
                           re.IGNORECASE|re.DOTALL)
 
@@ -337,8 +339,14 @@ class GnomoliciousApplet(gnomeapplet.Applet):
         logger.info('Received url: %s' % url)
         txt = urllib.urlopen(url).read()
         title_match = self.title_re.search(txt)
+        charset_match = self.charset_re.search(txt)
+        if charset_match:
+            charset = charset_match.groupdict()['enc']
+        else:
+            charset = 'utf-8'
         if title_match:
-            title = ' '.join(title_match.groupdict()['title'].split())
+            title = unicode(' '.join(title_match.groupdict()['title'].split()),
+                            charset)
         else:
             title = ''
         self.postwin.entries['url'].set_text(url)
